//
//  NSGeometryAdditions.h
//  AraeliumFoundation
//
//  Created by Seth Willits on 5/21/09.
//  Copyright 2009 Araelium Group. All rights reserved.
//

#import <Cocoa/Cocoa.h>


#if MAC_OS_X_VERSION_10_4 >= MAC_OS_X_VERSION_MAX_ALLOWED

static inline NSRect NSRectFromCGRect(CGRect cgrect) {
	return (*(NSRect *)(void *)&(cgrect));
}

static inline CGRect NSRectToCGRect(NSRect nsrect) {
	return (*(CGRect *)(void *)&(nsrect));
}

static inline NSPoint NSPointFromCGPoint(CGPoint cgpoint) {
	return (*(NSPoint *)(void *)&(cgpoint));
}

static inline CGPoint NSPointToCGPoint(NSPoint nspoint) {
	return (*(CGPoint *)(void *)&(nspoint));
}

static inline NSSize NSSizeFromCGSize(CGSize cgsize) {
	return (*(NSSize *)(void *)&(cgsize));
}

static inline CGSize NSSizeToCGSize(NSSize nssize) {
	return (*(CGSize *)(void *)&(nssize));
}

static inline NSString * NSStringFromRectArray(NSArray * array)
{
	NSString * string = @"";
	NSEnumerator * enumerator = [array objectEnumerator];
	NSValue * rectValue = nil;
	
	while (rectValue = [enumerator nextObject]) {
		string = [string stringByAppendingString:NSStringFromRect([rectValue rectValue])];
		string = [string stringByAppendingString:@"\n"];
	}
	
	return string;
}

#endif


static inline NSPoint NSRectCenter(NSRect rect)
{
	return NSMakePoint(NSMidX(rect), NSMidY(rect));
}



static inline NSRect NSRectCenteredOnPoint(NSPoint point, CGFloat width, CGFloat height)
{
	return NSMakeRect(point.x - floor(width / 2.0), point.y - floor(height / 2.0), width, height);
}


static inline NSRect NSRectScaledToFitInRect(NSSize scaleThisSize, NSRect toFitInThisRect)
{
	NSRect maxFrame = toFitInThisRect;
	NSSize size = scaleThisSize;
	CGFloat ratio = size.width / size.height;
	
	// Scale based on width
	if (size.width > maxFrame.size.width) {
		size.width = maxFrame.size.width;
		size.height = floor(size.width / ratio);
	}
	
	// Scale based on height
	if (size.height > maxFrame.size.height) {
		size.height = maxFrame.size.height;
		size.width = floor(size.height * ratio);
	}
	
	return NSMakeRect(toFitInThisRect.origin.x, toFitInThisRect.origin.y, size.width, size.height);
}


static inline NSRect NSRectCenteredInRect(NSRect centerMe, NSRect inThisRect)
{
	return NSRectCenteredOnPoint(NSMakePoint(NSMidX(inThisRect), NSMidY(inThisRect)), centerMe.size.width, centerMe.size.height);
}


static inline NSRect NSRectFitInRect(NSRect child, NSRect parent)
{
	// No bigger than the screen
	if (child.size.width  > parent.size.width)  child.size.width  = parent.size.width;
	if (child.size.height > parent.size.height) child.size.height = parent.size.height;
	
	// Move on to screen
	if (NSMinX(child) < NSMinX(parent)) child.origin.x += NSMinX(parent) - NSMinX(child);
	if (NSMinY(child) < NSMinY(parent)) child.origin.y += NSMinY(parent) - NSMinY(child);
	if (NSMaxX(child) > NSMaxX(parent)) child.origin.x -= NSMaxX(child) - NSMaxX(parent);
	if (NSMaxY(child) > NSMaxY(parent)) child.origin.y -= NSMaxY(child) - NSMaxY(parent);
	
	return child;
}



static inline NSRect NSRectIntegerized(NSRect rect)
{
	return NSMakeRect(round(rect.origin.x), round(rect.origin.y), round(rect.size.width), round(rect.size.height));
}


